@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 1996-2017 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -26,30 +29,25 @@ along with Octave; see the file COPYING.  If not, see
 
 #include <string>
 
-#include <iomanip>
-#include <iostream>
-
 #include "Quad.h"
 #include "lo-mappers.h"
 
 #include "defun.h"
 #include "error.h"
 #include "errwarn.h"
+#include "interpreter-private.h"
 #include "pager.h"
+#include "parse.h"
+#include "ov.h"
 #include "ovl.h"
-#include "ov-fcn.h"
 #include "unwind-prot.h"
 #include "utils.h"
 #include "variables.h"
 
 #include "Quad-opts.cc"
 
-#if defined (quad)
-#  undef quad
-#endif
-
 // Global pointer for user defined function required by quadrature functions.
-static octave_function *quad_fcn;
+static octave_value quad_fcn;
 
 // Have we warned about imaginary values returned from user function?
 static bool warned_imaginary = false;
@@ -65,13 +63,13 @@ quad_user_function (double x)
   octave_value_list args;
   args(0) = x;
 
-  if (quad_fcn)
+  if (quad_fcn.is_defined ())
     {
       octave_value_list tmp;
 
       try
         {
-          tmp = quad_fcn->do_multi_index_op (1, args);
+          tmp = octave::feval (quad_fcn, args, 1);
         }
       catch (octave::execution_exception& e)
         {
@@ -81,7 +79,7 @@ quad_user_function (double x)
       if (! tmp.length () || ! tmp(0).is_defined ())
         err_user_supplied_eval ("quad");
 
-      if (! warned_imaginary && tmp(0).is_complex_type ())
+      if (! warned_imaginary && tmp(0).iscomplex ())
         {
           warning ("quad: ignoring imaginary part returned from user-supplied function");
           warned_imaginary = true;
@@ -101,13 +99,13 @@ quad_float_user_function (float x)
   octave_value_list args;
   args(0) = x;
 
-  if (quad_fcn)
+  if (quad_fcn.is_defined ())
     {
       octave_value_list tmp;
 
       try
         {
-          tmp = quad_fcn->do_multi_index_op (1, args);
+          tmp = octave::feval (quad_fcn, args, 1);
         }
       catch (octave::execution_exception& e)
         {
@@ -117,7 +115,7 @@ quad_float_user_function (float x)
       if (! tmp.length () || ! tmp(0).is_defined ())
         err_user_supplied_eval ("quad");
 
-      if (! warned_imaginary && tmp(0).is_complex_type ())
+      if (! warned_imaginary && tmp(0).iscomplex ())
         {
           warning ("quad: ignoring imaginary part returned from user-supplied function");
           warned_imaginary = true;
@@ -129,8 +127,8 @@ quad_float_user_function (float x)
   return retval;
 }
 
-DEFUN (quad, args, ,
-       doc: /* -*- texinfo -*-
+DEFMETHODX ("quad", Fquad, interp, args, ,
+            doc: /* -*- texinfo -*-
 @deftypefn  {} {@var{q} =} quad (@var{f}, @var{a}, @var{b})
 @deftypefnx {} {@var{q} =} quad (@var{f}, @var{a}, @var{b}, @var{tol})
 @deftypefnx {} {@var{q} =} quad (@var{f}, @var{a}, @var{b}, @var{tol}, @var{sing})
@@ -151,7 +149,7 @@ absolute tolerance, and the second element is the desired relative
 tolerance.  To choose a relative test only, set the absolute
 tolerance to zero.  To choose an absolute test only, set the relative
 tolerance to zero.  Both tolerances default to @code{sqrt (eps)} or
-approximately @math{1.5e^{-8}}.
+approximately 1.5e-8.
 
 The optional argument @var{sing} is a vector of values at which the
 integrand is known to be singular.
@@ -190,23 +188,7 @@ variable by routines @code{dblquad} and @code{triplequad}.
   if (call_depth > 1)
     error ("quad: invalid recursive call");
 
-  std::string fcn_name;
-
-  if (args(0).is_function_handle () || args(0).is_inline_function ())
-    quad_fcn = args(0).function_value ();
-  else
-    {
-      fcn_name = unique_symbol_name ("__quad_fcn__");
-      std::string fname = "function y = ";
-      fname.append (fcn_name);
-      fname.append ("(x) y = ");
-      quad_fcn = extract_function (args(0), "quad", fcn_name, fname,
-                                   "; endfunction");
-      frame.add_fcn (clear_function, fcn_name);
-    }
-
-  if (! quad_fcn)
-    error ("quad: FCN argument is not a valid function name or handle");
+  quad_fcn = octave::get_function_handle (interp, args(0), "x");
 
   octave_value_list retval;
 
@@ -254,6 +236,7 @@ variable by routines @code{dblquad} and @code{triplequad}.
           have_sing = true;
 
           sing = args(4).xfloat_vector_value ("quad: fifth argument SING must be a vector of singularities");
+          OCTAVE_FALLTHROUGH;
 
         case 4:
           tol = args(3).xfloat_vector_value ("quad: TOL must be a 1 or 2-element vector");
@@ -262,6 +245,7 @@ variable by routines @code{dblquad} and @code{triplequad}.
             {
             case 2:
               quad_opts.set_single_precision_relative_tolerance (tol (1));
+              OCTAVE_FALLTHROUGH;
 
             case 1:
               quad_opts.set_single_precision_absolute_tolerance (tol (0));
@@ -270,6 +254,7 @@ variable by routines @code{dblquad} and @code{triplequad}.
             default:
               error ("quad: TOL must be a 1 or 2-element vector");
             }
+          OCTAVE_FALLTHROUGH;
 
         case 3:
           if (indefinite)
@@ -346,7 +331,8 @@ variable by routines @code{dblquad} and @code{triplequad}.
 
           have_sing = true;
 
-          sing = args(4).vector_value ("quad: fifth argument SING must be a vector of singularities");
+          sing = args(4).xvector_value ("quad: fifth argument SING must be a vector of singularities");
+          OCTAVE_FALLTHROUGH;
 
         case 4:
           tol = args(3).xvector_value ("quad: TOL must be a 1 or 2-element vector");
@@ -355,6 +341,7 @@ variable by routines @code{dblquad} and @code{triplequad}.
             {
             case 2:
               quad_opts.set_relative_tolerance (tol (1));
+              OCTAVE_FALLTHROUGH;
 
             case 1:
               quad_opts.set_absolute_tolerance (tol (0));
@@ -363,6 +350,7 @@ variable by routines @code{dblquad} and @code{triplequad}.
             default:
               error ("quad: TOL must be a 1 or 2-element vector");
             }
+          OCTAVE_FALLTHROUGH;
 
         case 3:
           if (indefinite)
@@ -396,9 +384,6 @@ variable by routines @code{dblquad} and @code{triplequad}.
       retval = ovl (val, ier, nfun, abserr);
     }
 
-  if (fcn_name.length ())
-    clear_function (fcn_name);
-
   return retval;
 }
 
@@ -431,6 +416,33 @@ variable by routines @code{dblquad} and @code{triplequad}.
 %! assert (v, 1.98194120273598, sqrt (eps));
 %! assert (nfun > 0);
 
+%!test
+%!  [v, ier, nfun, err] = quad (@__f, 0.001, 3);
+%! assert (ier == 0 || ier == 1);
+%! assert (v, 1.98194120273598, sqrt (eps));
+%! assert (nfun > 0);
+
+%!test
+%!  fstr = "x .* sin (1 ./ x) .* sqrt (abs (1 - x))";
+%!  [v, ier, nfun, err] = quad (fstr, 0.001, 3);
+%! assert (ier == 0 || ier == 1);
+%! assert (v, 1.98194120273598, sqrt (eps));
+%! assert (nfun > 0);
+
+%!test
+%!  anon_fcn = @(x) x .* sin (1 ./ x) .* sqrt (abs (1 - x));
+%!  [v, ier, nfun, err] = quad (anon_fcn, 0.001, 3);
+%! assert (ier == 0 || ier == 1);
+%! assert (v, 1.98194120273598, sqrt (eps));
+%! assert (nfun > 0);
+
+%!test
+%!  inline_fcn = inline ("x .* sin (1 ./ x) .* sqrt (abs (1 - x))", "x");
+%!  [v, ier, nfun, err] = quad (inline_fcn, 0.001, 3);
+%! assert (ier == 0 || ier == 1);
+%! assert (v, 1.98194120273598, sqrt (eps));
+%! assert (nfun > 0);
+
 %!test
 %!  [v, ier, nfun, err] = quad ("__f", single (0.001), single (3));
 %! assert (ier == 0 || ier == 1);
