@@ -1,5 +1,9 @@
-## Copyright (C) 2004-2018 Piotr Krzyzanowski
-## Copyright (C) 2016-2018 Cristiano Dorigo
+########################################################################
+##
+## Copyright (C) 2004-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -16,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{x} =} pcg (@var{A}, @var{b}, @var{tol}, @var{maxit}, @var{m1}, @var{m2}, @var{x0}, @dots{})
@@ -253,11 +259,6 @@
 ## @seealso{sparse, pcr, gmres, bicg, bicgstab, cgs}
 ## @end deftypefn
 
-## Author: Piotr Krzyzanowski <piotr.krzyzanowski@mimuw.edu.pl>
-## Modified by: Vittoria Rezzonico <vittoria.rezzonico@epfl.ch>
-##  - Add the ability to provide the pre-conditioner as two separate matrices
-
-
 function [x_min, flag, relres, iter_min, resvec, eigest] =...
          pcg (A, b, tol = [], maxit = [], M1 = [], M2 = [], x0 = [], varargin)
 
@@ -356,7 +357,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
     x += alpha * p;
     r -= alpha * w;
     resvec(iter, 1) = norm (r);
-    ## Chek if the iterated has minimum residual
+    ## Check if the iterated has minimum residual
     if (resvec (iter,1) <= resvec (iter_min + 1,1))
       x_min = x;
       iter_min = iter - 1;
@@ -410,7 +411,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
   elseif (resvec (1, 1) == 0)
     relres = 0;
   else
-    relres = resvec(iter_min+1, 1) ./ resvec(1, 1);
+    relres = resvec(iter_min+1, 1) ./ b_norm;
   endif
 
   iter -= 2; # compatibility
@@ -551,10 +552,10 @@ endfunction
 %!test
 %! ## solve tridiagonal system, do not converge in default 20 iterations
 %! N = 100;
-%! A = zeros (N, N);
-%! for i = 1 : N - 1  # form 1-D Laplacian matrix
-%!   A(i:i+1, i:i+1) = [2 -1; -1 2];
-%! endfor
+%! ## Form 1-D Laplacian matrix
+%! A = 2 * eye (N,N);
+%! A(2:(N+1):end) = -1;
+%! A((N+1):(N+1):end) = -1;
 %! b = ones (N, 1);
 %! [x, flag, relres, iter, resvec, eigest] = pcg (A, b, 1e-12);
 %! assert (flag);
@@ -564,15 +565,15 @@ endfunction
 %! ## solve tridiagonal system with "perfect" preconditioner which converges
 %! ## in one iteration, so the eigest does not work and issues a warning.
 %! N = 100;
-%! A = zeros (N, N);
-%! for i = 1 : N - 1  # form 1-D Laplacian matrix
-%!   A(i:i+1, i:i+1) = [2 -1; -1 2];
-%! endfor
+%! ## Form 1-D Laplacian matrix
+%! A = 2 * eye (N,N);
+%! A(2:(N+1):end) = -1;
+%! A((N+1):(N+1):end) = -1;
 %! b = ones (N, 1);
 %! [x, flag, relres, iter, resvec, eigest] = pcg (A, b, [], [], A, [], b);
 %! assert (flag, 0);
 %! assert (norm (b - A*x) / norm (b), 0, 1e-6);
-%! 
+%!
 %! assert (isnan (eigest), isnan ([NaN, NaN]));
 
 %!test
@@ -671,3 +672,13 @@ endfunction
 %! [x, flag, relres] = pcg (A, b, [], 2, M);
 %! assert (norm (b - A * x) / norm (b), relres,  8 * eps);
 
+%!test <*59776>
+%! A = [ 1.00000000  -0.00054274  -0.00066848;
+%!      -0.00054274   1.00000000  -0.00060330;
+%!      -0.00066848  -0.00060330   1.00000000];
+%! b = [1 1 1]';
+%! [x, flag, relres, iter, resvec] = pcg (A, b, 1e-6, 4, [], [], [1; 1; 1]);
+%! assert (flag, 0);
+%! assert (relres, resvec(2) / norm (b));
+%! assert (iter, 1);
+
